/*
 * main.c
 *
 *  Created on: Jan 13, 2012
 *      Author: mbuckley
 */

#include <stdio.h>
#include <math.h>
#include <string.h>

#include <pololu/orangutan.h>

int min_x;
int min_y;
int min_z;
int max_x;
int max_y;
int max_z;
int compass_x;
int compass_y;
int compass_z;

void setup(void);
void loop(void);

int main(void)
{
	setup();
	while (1)
	{
		loop();
	}

	return 0;
}

void i2c_start()
{
	TWCR = (1 << TWINT) | (1 << TWSTA) | (1 << TWEN); // send start condition
	while (!(TWCR & (1 << TWINT)))
		;
}

void i2c_write_byte(char byte)
{
	TWDR = byte;
	TWCR = (1 << TWINT) | (1 << TWEN); // start address transmission
	while (!(TWCR & (1 << TWINT)))
		;
}

char i2c_read_byte()
{
	TWCR = (1 << TWINT) | (1 << TWEA) | (1 << TWEN); // start data reception, transmit ACK
	while (!(TWCR & (1 << TWINT)))
		;
	return TWDR;
}

char i2c_read_last_byte()
{
	TWCR = (1 << TWINT) | (1 << TWEN); // start data reception
	while (!(TWCR & (1 << TWINT)))
		;
	return TWDR;
}

void i2c_stop()
{
	TWCR = (1 << TWINT) | (1 << TWSTO) | (1 << TWEN); // send stop condition
}

void I2C_Init()
{
	DDRC = 0; // all inputs
	PORTC = (1 << PORTC4) | (1 << PORTC5); // enable pull-ups on SDA and SCL, respectively

	TWSR = 0; // clear bit-rate prescale bits
	TWBR = 17; // produces an SCL frequency of 400 kHz with a 20 MHz CPU clock speed
}

void Read_Compass()
{
	// read magnetometer values
	i2c_start();
	i2c_write_byte(0x3C); // write mag
	i2c_write_byte(0x03); // OUTXH_M
	i2c_start(); // repeated start
	i2c_write_byte(0x3D); // read mag
	unsigned char mxh = i2c_read_byte();
	unsigned char mxl = i2c_read_byte();
	unsigned char mzh = i2c_read_byte();	// LSM303DLM z is before y
	unsigned char mzl = i2c_read_byte();
	unsigned char myh = i2c_read_byte();
	unsigned char myl = i2c_read_last_byte();
	i2c_stop();

	compass_x = (mxh << 8 | mxl);
	compass_y = (myh << 8 | myl);
	compass_z = (mzh << 8 | mzl);
}

void Compass_Init()
{
//	compass.writeMagReg(LSM303_MR_REG_M, 0x00); // continuous conversion mode
	// 15 Hz default
	i2c_start();
	i2c_write_byte(0x3C); // write mag
	i2c_write_byte(0x02); // MR_REG_M
	i2c_write_byte(0x00); // continuous conversion mode
	i2c_stop();
}

void setup()
{
	I2C_Init();
	Compass_Init();
}

int min(int a, int b)
{
	if (a < b)
	{
		return a;
	}
	return b;
}

int max(int a, int b)
{
	if (a > b)
	{
		return a;
	}
	return b;
}

void loop()
{
	Read_Compass(); // Read I2C magnetometer

	min_x = min(min_x, compass_x);
	min_y = min(min_y, compass_y);
	min_z = min(min_z, compass_z);

	max_x = max(max_x, compass_x);
	max_y = max(max_y, compass_y);
	max_z = max(max_z, compass_z);

	char str[100];

	sprintf(str, "M min X: %3d Y: %3d Z: %3d M max X: %3d Y: %3d Z: %3d\r\n",
			min_x, min_y, min_z, max_x, max_y, max_z);
	serial_send_blocking(USB_COMM, str, strlen(str));

	delay(100);
}
