/*

 MinIMU-9-Arduino-AHRS
 Pololu MinIMU-9 + Arduino AHRS (Attitude and Heading Reference System)

 Copyright (c) 2011 Pololu Corporation.
 http://www.pololu.com/

 MinIMU-9-Arduino-AHRS is based on sf9domahrs by Doug Weibel and Jose Julio:
 http://code.google.com/p/sf9domahrs/

 sf9domahrs is based on ArduIMU v1.5 by Jordi Munoz and William Premerlani, Jose
 Julio and Doug Weibel:
 http://code.google.com/p/ardu-imu/

 MinIMU-9-Arduino-AHRS is free software: you can redistribute it and/or modify it
 under the terms of the GNU Lesser General Public License as published by the
 Free Software Foundation, either version 3 of the License, or (at your option)
 any later version.

 MinIMU-9-Arduino-AHRS is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for
 more details.

 You should have received a copy of the GNU Lesser General Public License along
 with MinIMU-9-Arduino-AHRS. If not, see <http://www.gnu.org/licenses/>.

 */
#include <pololu/orangutan.h>

#include "MinIMU9AHRS.h"

void i2c_start()
{
	TWCR = (1 << TWINT) | (1 << TWSTA) | (1 << TWEN); // send start condition
	while (!(TWCR & (1 << TWINT)))
		;
}

void i2c_write_byte(char byte)
{
	TWDR = byte;
	TWCR = (1 << TWINT) | (1 << TWEN); // start address transmission
	while (!(TWCR & (1 << TWINT)))
		;
}

char i2c_read_byte()
{
	TWCR = (1 << TWINT) | (1 << TWEA) | (1 << TWEN); // start data reception, transmit ACK
	while (!(TWCR & (1 << TWINT)))
		;
	return TWDR;
}

char i2c_read_last_byte()
{
	TWCR = (1 << TWINT) | (1 << TWEN); // start data reception
	while (!(TWCR & (1 << TWINT)))
		;
	return TWDR;
}

void i2c_stop()
{
	TWCR = (1 << TWINT) | (1 << TWSTO) | (1 << TWEN); // send stop condition
}

void I2C_Init()
{
	DDRC = 0; // all inputs
	PORTC = (1 << PORTC4) | (1 << PORTC5); // enable pull-ups on SDA and SCL, respectively

	TWSR = 0; // clear bit-rate prescale bits
	TWBR = 17; // produces an SCL frequency of 400 kHz with a 20 MHz CPU clock speed
}

void Gyro_Init()
{
//	gyro.writeReg(L3G4200D_CTRL_REG1, 0x0F); // normal power mode, all axes enabled, 100 Hz
//	gyro.writeReg(L3G4200D_CTRL_REG4, 0x20); // 2000 dps full scale
	i2c_start();
	i2c_write_byte(0xD2); // write gyro
	i2c_write_byte(0x20); // CTRL_REG1_A
	i2c_write_byte(0x0F); // normal power mode, 1000 Hz data rate, all axes enabled
	i2c_stop();

	i2c_start();
	i2c_write_byte(0xD2); // write gyro
	i2c_write_byte(0x23); // CTRL_REG4_A
	i2c_write_byte(0x20); // 2000 dps full scale
	i2c_stop();
}

void Read_Gyro()
{
	// read gyro values
	i2c_start();
	i2c_write_byte(0xD2); 	// write gyro
	i2c_write_byte(0xa8); 	// OUT_X_L_A, MSB set to enable auto-increment
	i2c_start(); 			// repeated start
	i2c_write_byte(0xD3); 	// read gyro
	unsigned char gxl = i2c_read_byte();
	unsigned char gxh = i2c_read_byte();
	unsigned char gyl = i2c_read_byte();
	unsigned char gyh = i2c_read_byte();
	unsigned char gzl = i2c_read_byte();
	unsigned char gzh = i2c_read_last_byte();
	i2c_stop();

	AN[0] = gxh << 8 | gxl;
	AN[1] = gyh << 8 | gyl;
	AN[2] = gzh << 8 | gzl;
	gyro_x = SENSOR_SIGN[0] * (AN[0] - AN_OFFSET[0]);
	gyro_y = SENSOR_SIGN[1] * (AN[1] - AN_OFFSET[1]);
	gyro_z = SENSOR_SIGN[2] * (AN[2] - AN_OFFSET[2]);
}

void Accel_Init()
{
//	compass.writeAccReg(LSM303_CTRL_REG1_A, 0x27); // normal power mode, all axes enabled, 50 Hz
//	compass.writeAccReg(LSM303_CTRL_REG4_A, 0x30); // 8 g full scale
	//enable accelerometer
	i2c_start();
	i2c_write_byte(0x30); // write acc
	i2c_write_byte(0x20); // CTRL_REG1_A
	i2c_write_byte(0x27); // normal power mode, 50 Hz data rate, all axes enabled
	i2c_stop();

	i2c_start();
	i2c_write_byte(0x30); // write acc
	i2c_write_byte(0x23); // CTRL_REG4_A
	i2c_write_byte(0x30); // 8 g full scale
	i2c_stop();
}

// Reads x,y and z accelerometer registers
void Read_Accel()
{
	// read accelerometer values
	i2c_start();
	i2c_write_byte(0x30); 	// write acc
	i2c_write_byte(0xa8); 	// OUT_X_L_A, MSB set to enable auto-increment
	i2c_start(); 			// repeated start
	i2c_write_byte(0x31); 	// read acc
	unsigned char axl = i2c_read_byte();
	unsigned char axh = i2c_read_byte();
	unsigned char ayl = i2c_read_byte();
	unsigned char ayh = i2c_read_byte();
	unsigned char azl = i2c_read_byte();
	unsigned char azh = i2c_read_last_byte();
	i2c_stop();

	AN[3] = (axh << 8 | axl) >> 4;
	AN[4] = (ayh << 8 | ayl) >> 4;
	AN[5] = (azh << 8 | azl) >> 4;
	accel_x = SENSOR_SIGN[3] * (AN[3] - AN_OFFSET[3]);
	accel_y = SENSOR_SIGN[4] * (AN[4] - AN_OFFSET[4]);
	accel_z = SENSOR_SIGN[5] * (AN[5] - AN_OFFSET[5]);
}

void Compass_Init()
{
//	compass.writeMagReg(LSM303_MR_REG_M, 0x00); // continuous conversion mode
	// 15 Hz default
	i2c_start();
	i2c_write_byte(0x3C); // write mag
	i2c_write_byte(0x02); // MR_REG_M
	i2c_write_byte(0x00); // continuous conversion mode
	i2c_stop();
}

void Read_Compass()
{
	// read magnetometer values
	i2c_start();
	i2c_write_byte(0x3C); // write mag
	i2c_write_byte(0x03); // OUTXH_M
	i2c_start(); // repeated start
	i2c_write_byte(0x3D); // read mag
	unsigned char mxh = i2c_read_byte();
	unsigned char mxl = i2c_read_byte();
	unsigned char mzh = i2c_read_byte();	// LSM303DLM z is before y
	unsigned char mzl = i2c_read_byte();
	unsigned char myh = i2c_read_byte();
	unsigned char myl = i2c_read_last_byte();
	i2c_stop();

	magnetom_x = SENSOR_SIGN[6] * (mxh << 8 | mxl);
	magnetom_y = SENSOR_SIGN[7] * (myh << 8 | myl);
	magnetom_z = SENSOR_SIGN[8] * (mzh << 8 | mzl);
}

